@@ -1,5 +1,5 @@
 /* libFLAC++ - Free Lossless Audio Codec library
- * Copyright (C) 2002,2003,2004  Josh Coalson
+ * Copyright (C) 2002,2003,2004,2005  Josh Coalson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -169,7 +169,7 @@ namespace FLAC {
 		bool Stream::set_metadata(FLAC::Metadata::Prototype **metadata, unsigned num_blocks)
 		{
 			FLAC__ASSERT(is_valid());
-#ifdef _MSC_VER
+#if (defined _MSC_VER) || (defined __SUNPRO_CC)
 			// MSVC++ can't handle:
 			// ::FLAC__StreamMetadata *m[num_blocks];
 			// so we do this ugly workaround
@@ -181,7 +181,7 @@ namespace FLAC {
 				// we can get away with this since we know the encoder will only correct the is_last flags
 				m[i] = const_cast< ::FLAC__StreamMetadata*>((::FLAC__StreamMetadata*)metadata[i]);
 			}
-#ifdef _MSC_VER
+#if (defined _MSC_VER) || (defined __SUNPRO_CC)
 			// complete the hack
 			const bool ok = (bool)::FLAC__stream_encoder_set_metadata(encoder_, m, num_blocks);
 			delete [] m;
@@ -356,5 +356,5 @@ namespace FLAC {
 			instance->metadata_callback(metadata);
 		}
 
-	};
-};
+	}
+}
